"""
data_processor.py

This module contains the DataProcessor class for calculating and printing descriptive statistics.
"""
import logging


class DataProcessor:
    """
    A class responsible for processing and analyzing the dataset.

    Example usage:
        >>> import pandas as pd
        >>> data = pd.DataFrame({'A': [1, 2, 3], 'B': [4, 5, 6]})
        >>> processor = DataProcessor(data)
        >>> list(processor)
        ['A', 'B']
        >>> processor.get_statistics()  # doctest: +NORMALIZE_WHITESPACE
        {'A': {'mean': 2.0, 'median': 2.0, 'mode': 1, 'range': 2.0},
         'B': {'mean': 5.0, 'median': 5.0, 'mode': 4, 'range': 2.0}}
    """

    def __init__(self, data):
        """
        Initializes the DataProcessor with a dataset.

        Parameters:
            data (DataFrame): The pandas DataFrame containing the dataset.

        Example:
            >>> import pandas as pd
            >>> data = pd.DataFrame({'A': [1, 2, 3], 'B': [4, 5, 6]})
            >>> processor = DataProcessor(data)
            >>> processor.data.equals(data)
            True
        """
        self.data = data
        self.logger = logging.getLogger(__name__)

        # Logging
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
        )

    def __iter__(self):
        """
        Returns an iterator for numerical columns in the dataset.

        Yields:
            str: The name of the numerical column

        Example:
            >>> import pandas as pd
            >>> data = pd.DataFrame({'A': [1, 2, 3], 'B': [4, 5, 6], 'C': ['a', 'b', 'c']})
            >>> processor = DataProcessor(data)
            >>> list(processor)
            ['A', 'B']
        """
        numerical_columns = self.data.select_dtypes(include="number").columns
        if not numerical_columns.any():
            self.logger.warning("No numerical columns found in the dataset.")
            return iter([])  # Return an empty iterator if no numerical columns
        return iter(numerical_columns)

    def get_statistics(self):
        """
        Calculates and returns descriptive statistics for numerical columns.

        Returns:
            dict: A dictionary with mean, median, mode, and range for each column.

        Example:
            >>> import pandas as pd
            >>> data = pd.DataFrame({'A': [1, 2, 3], 'B': [4, 5, 6]})
            >>> processor = DataProcessor(data)
            >>> processor.get_statistics()  # doctest: +NORMALIZE_WHITESPACE
            {'A': {'mean': 2.0, 'median': 2.0, 'mode': 1, 'range': 2.0},
             'B': {'mean': 5.0, 'median': 5.0, 'mode': 4, 'range': 2.0}}
        """
        stats = {}
        for column in self:
            try:
                self.logger.info(f"Processing statistics for column: {column}")
                col_data = self.data[column].dropna()  # Exclude NaN values

                if col_data.empty:  # Check if column is empty after removing NaN
                    self.logger.warning(f"Column {column} contains no valid data.")
                    stats[column] = {
                        "error": "No valid data",
                    }
                else:
                    stats[column] = {
                        'mean': float(col_data.mean()),
                        'median': float(col_data.median()),
                        'mode': int(col_data.mode()[0]) if not col_data.mode().empty else None,
                        'range': float(col_data.max() - col_data.min()),
                    }
            except Exception as e:
                self.logger.error(f"Error processing column {column}: {e}")
                stats[column] = {"error": str(e)}
        return stats

    def print_statistics(self):
        """
        Prints general descriptive statistics for the dataset.

        Example:
            >>> import pandas as pd
            >>> data = pd.DataFrame({'A': [1, 2, 3], 'B': [4, 5, 6]})
            >>> processor = DataProcessor(data)
            >>> processor.print_statistics()  # doctest: +ELLIPSIS
            Descriptive Statistics Summary:
                     A    B
            count  3.0  3.0
            mean   2.0  5.0
            std    1.0  1.0
            min    1.0  4.0
            25%    1.5  4.5
            50%    2.0  5.0
            75%    2.5  5.5
            max    3.0  6.0
        """

        self.logger.info("Printing general descriptive statistics for the dataset.")
        try:
            print("Descriptive Statistics Summary:")
            print(self.data.describe())
        except Exception as e:
            self.logger.error(f"Error while printing statistics: {e}")
            raise
