import unittest
from unittest.mock import patch
import pandas as pd
from .data_loader2 import DataLoader
from .data_processor import DataProcessor
from .weatherData_analysis import WeatherAnalysis


class TestDataLoader(unittest.TestCase):
    def setUp(self):
        self.file_path = "Phase_5/src/weather_analysis/weatherTestData.csv"
        self.chunk_size = 2
        self.loader = DataLoader(self.file_path, self.chunk_size)

    @patch("pandas.read_csv")
    def test_load_data_success(self, mock_read_csv):
        # Mock the pandas read_csv method
        mock_read_csv.return_value = [
            pd.DataFrame({"A": [1, 2], "B": [3, 4]}),
            pd.DataFrame({"A": [5, 6], "B": [7, 8]}),
        ]

        chunks = list(self.loader.load_data())

        self.assertEqual(len(chunks), 2)
        self.assertTrue((chunks[0]["A"] == [1, 2]).all())
        self.assertTrue((chunks[1]["A"] == [5, 6]).all())

    @patch("pandas.read_csv", side_effect=FileNotFoundError)
    def test_load_data_file_not_found(self, mock_read_csv):
        with self.assertRaises(FileNotFoundError):
            list(self.loader.load_data())


class TestDataProcessor(unittest.TestCase):
    def setUp(self):
        self.data = pd.DataFrame({
            "A": [1, 2, 3],
            "B": [4, 5, 6],
            "C": ["x", "y", "z"],
        })
        self.processor = DataProcessor(self.data)

    def test_iter(self):
        columns = list(self.processor)
        self.assertEqual(columns, ["A", "B"])

    def test_get_statistics(self):
        stats = self.processor.get_statistics()
        self.assertIn("A", stats)
        self.assertIn("B", stats)
        self.assertEqual(stats["A"].get("mean"), 2)
        self.assertEqual(stats["B"].get("mean"), 5)

    def test_get_statistics_with_empty_column(self):
        self.processor.data["A"] = pd.Series(dtype="float64")

        stats = self.processor.get_statistics()

        self.assertIn("A", stats)  # Ensure "A" exists in stats
        self.assertIn("error", stats["A"])  # Check that "error" is in stats["A"]


class TestWeatherAnalysis(unittest.TestCase):
    @patch('weather_analysis.data_loader2.pd.read_csv')
    def test_run_analysis(self, mock_read_csv):
        # Create a mock DataFrame to simulate data loading
        dummy_data = pd.DataFrame({'A': [1, 2], 'B': [3, 4]})
        mock_read_csv.return_value = [dummy_data]  # Simulate a chunk generator

        # Instantiate and run your analysis
        analysis = WeatherAnalysis(file_path='dummy_path.csv')
        analysis.run_analysis()

        # Verify that read_csv was called correctly
        mock_read_csv.assert_called_with('dummy_path.csv', chunksize=1000)


if __name__ == "__main__":
    unittest.main()
